import 'package:floative/factory/floative_factory.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  FloativeFactory factory;

  setUpAll(() {
    factory = FloativeFactory.instance;
  });

  test("test the register of singleton in normal conditions", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);
  });

  test("test the register of a LAZY singleton in normal conditions", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst, isLazy: true, overrideInstance: true);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);
  });

  test("try to get a singleton that was not registered", () {
    factory.reset();
    var foo = factory.get<Foo>();

    expect(foo, null);
  });

  test("Register a factory and get it as brand new", () {
    final foo = Foo();

    factory.registerFactory<Foo>(() => foo);

    expect(foo, isInstanceOf<Foo>());
  });

  test("reset the factory", () {
    final foo = Foo();
    final fooNew = FooNew();
    factory.registerFactory<Foo>(() => foo);
    factory.register<FooNew>(() => fooNew);

    factory.reset();

    var fooFromFactory = factory.get<Foo>();
    var fooNewFromFactory = factory.get<FooNew>();
    expect(fooFromFactory, null);
    expect(fooNewFromFactory, null);
  });

  test("test the remove of a factory", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);

    factory.dispose<Foo>();

    var fooAgain = factory.get<Foo>();

    expect(fooAgain, null);
  });

  test("test the args", () {
    factory.reset();
    factory.register<Foo>(() => Foo());

    var foo =
        factory.get<Foo>(args: {'test_key': 1, 'foo_new_instance': FooNew()});

    expect(foo.args['test_key'], 1);
    expect(foo.args['foo_new_instance'], isInstanceOf<FooNew>());
  });

  test("test register without type", () {
    factory.reset();

    var fooNew = FooNew();
    fooNew.value = "type test";
    factory.register(() => fooNew);
    var foo = Foo();
    foo.value = 1;
    factory.register(() => foo);

    var fooNewAfterGet = factory.get<FooNew>();
    var fooAfterGet = factory.get<Foo>();

    expect(fooNewAfterGet, isInstanceOf<FooNew>());
    expect(fooNewAfterGet.value, "type test");

    expect(fooAfterGet, isInstanceOf<Foo>());
    expect(fooAfterGet.value, 1);
  });
}

class Foo implements FactoryArgs {
  int value = 0;

  void increment() => value++;

  @override
  Map<String, dynamic> args = <String, dynamic>{};
}

class FooNew {
  String value = '';

  void giveMeValue() => value = "value of FooNew";
}
